@@ -72,6 +72,7 @@ public:
 
     bool   action_convert                     (void);
     bool   action_revert                      (void);
+    bool   action_cancel_all                  (void);
     bool   action_commit_follow_preference    (void);
     bool   action_commit_reverse_preference   (void);
     bool   action_commit_first_segment        (void);
@@ -123,6 +124,8 @@ public:
     bool   action_convert_to_half_katakana    (void);
     bool   action_convert_to_latin            (void);
     bool   action_convert_to_wide_latin       (void);
+    bool   action_convert_char_type_forward   (void);
+    bool   action_convert_char_type_backward  (void);
 
     bool   action_circle_input_mode           (void);
     bool   action_circle_typing_method        (void);
@@ -140,7 +143,11 @@ public:
 
 public:
     AnthyFactory *
-           get_factory                        (void) { return m_factory; };
+           get_factory                        (void);
+    TypingMethod
+           get_typing_method                  (void);
+    InputMode
+           get_input_mode                     (void);
 
 private:
     /* processing key event */
@@ -178,8 +185,7 @@ private:
 
     bool                  m_on_init;
 
-    /* for preedit */
-    Key2KanaTableSet      m_key2kana_tables;
+   /* for preedit */
     Preedit               m_preedit;
     bool                  m_preedit_string_visible;
 
@@ -196,9 +202,6 @@ private:
 
     /*  */
     ConversionMode        m_conv_mode;
-
-    /* FIXME! */
-    bool                  m_forwarding_thumb_shift_key;
 };
 #endif /* __SCIM_ANTHY_IMENGINE_H__ */
 /*
