@@ -60,38 +60,43 @@ class Key2KanaRule
 public:
     Key2KanaRule ();
     Key2KanaRule (String sequence,
-                  String result,
-                  String cont);
+                  const std::vector<String> &result);
     virtual ~Key2KanaRule ();
 
-    String get_sequence        (void) { return m_sequence; }
-    String get_result          (void) { return m_result; }
-    String get_continue_string (void) { return m_continue; }
+    String get_sequence        (void);
+    String get_result          (unsigned int idx);
 
     void   clear               (void);
 
     bool   is_empty            (void);
 
 private:
-    String m_sequence;
-    String m_result;
-    String m_continue;
+    String              m_sequence;
+    std::vector<String> m_result;
 };
 
 
 class Key2KanaTable
 {
 public:
-    Key2KanaTable (WideString name);
-    Key2KanaTable (WideString name,
-                   ConvRule *table);
+    Key2KanaTable (WideString  name);
+    Key2KanaTable (WideString  name,
+                   ConvRule   *table);
+    Key2KanaTable (WideString  name,
+                   NicolaRule *table);
     virtual ~Key2KanaTable ();
 
     Key2KanaRules & get_table   (void) { return m_rules; }
 
+    void            append_rule (String sequence,
+                                 const std::vector<String> &result);
     void            append_rule (String sequence,
                                  String result,
                                  String cont);
+    void            append_rule (String sequence,
+                                 String normal,
+                                 String left_shift,
+                                 String right_shift);
     void            clear       (void);
 
 private:
@@ -126,7 +131,7 @@ public:
          get_comma_style         (void) { return m_comma_style; }
 
 #if 0
-    // void set_use_consonant_table (bool use);
+    void set_use_consonant_table (bool use);
     void set_use_symbol_table    (bool use);
     void set_use_number_table    (bool use);
     void set_use_period_table    (bool use);
