@@ -43,16 +43,10 @@ typedef enum {
     SCIM_ANTHY_MODE_WIDE_LATIN,
 } InputMode;
 
-typedef enum {
-    SCIM_ANTHY_NONE_ON_PERIOD,
-    SCIM_ANTHY_CONVERT_ON_PERIOD,
-    SCIM_ANTHY_COMMIT_ON_PERIOD,
-} BehaviorOnPeriod;
-
 class Preedit
 {
 public:
-    Preedit (AnthyInstance &anthy, Key2KanaTableSet & tables);
+    Preedit (AnthyInstance &anthy);
     virtual ~Preedit ();
 
     // getting status
@@ -97,49 +91,42 @@ public:
 
     // manipulating the caret
     virtual unsigned int  get_caret_pos          (void);
-    virtual void          set_caret_pos          (unsigned int pos);
-    virtual void          move_caret             (int len);
+    virtual void          set_caret_pos          (unsigned int   pos);
+    virtual void          move_caret             (int            len);
 
     // clear all string
     virtual void          clear                  (void);
 
     // preference
-    virtual void          set_input_mode         (InputMode mode);
+    virtual void          set_input_mode         (InputMode      mode);
     virtual InputMode     get_input_mode         (void);
-    virtual void          set_typing_method      (TypingMethod method,
-                                                  Key2KanaTable *fund_table
-                                                  = NULL);
+    virtual void          set_typing_method      (TypingMethod   method);
     virtual TypingMethod  get_typing_method      (void);
-    virtual void          set_ten_key_type       (TenKeyType type);
-    virtual TenKeyType    get_ten_key_type       (void);
-    virtual void          set_behavior_on_period (BehaviorOnPeriod behavior);
-    virtual BehaviorOnPeriod
-                          get_behavior_on_period (void);
-    virtual void          set_allow_split_romaji (bool allow);
-    virtual bool          get_allow_split_romaji (void);
+    virtual void          set_period_style       (PeriodStyle    style);
+    virtual PeriodStyle   get_period_style       (void);
+    virtual void          set_comma_style        (CommaStyle     style);
+    virtual CommaStyle    get_comma_style        (void);
+    virtual void          set_symbol_width       (bool           half);
+    virtual bool          get_symbol_width       (void);
+    virtual void          set_number_width       (bool           half);
+    virtual bool          get_number_width       (void);
 
 private:
-    void          get_reading_substr             (WideString & substr,
-                                                  unsigned int start,
-                                                  unsigned int len,
-                                                  CandidateType type);
-    bool          is_comma_or_period             (const String & str);
+    void                  get_reading_substr     (WideString   & substr,
+                                                  unsigned int   start,
+                                                  unsigned int   len,
+                                                  CandidateType  type);
+    bool                  is_comma_or_period     (const String & str);
 
 private:
     AnthyInstance    &m_anthy;
 
     // converter objects
-    Key2KanaTableSet &m_key2kana_tables;
     Reading           m_reading;
     Conversion        m_conversion;
 
     // mode flags
     InputMode         m_input_mode;
-    BehaviorOnPeriod  m_behavior_on_period;
-    bool              m_romaji_allow_split;
-
-    unsigned int      m_preedit_fg_color;
-    unsigned int      m_preedit_bg_color;
 };
 
 }
