@@ -210,10 +210,10 @@ IntConfigData config_int_common [] =
         SCIM_ANTHY_CONFIG_NICOLA_TIME_DEFAULT,
         SCIM_ANTHY_CONFIG_NICOLA_TIME_DEFAULT,
         0, 10000, 1,
-        N_(""),
+        N_("NICOLA time:"),
         N_("[msec]"),
         NULL,
-        N_(""),
+        N_("NICOLA time"),
         NULL,
         false,
     },
@@ -322,6 +322,39 @@ StringConfigData config_string_common [] =
         NULL,
         false,
     },
+    {
+        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY,
+        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY_DEFAULT,
+        N_("_Commit:"),
+        N_("Select commit keys"),
+        N_("The key events to commit the preedit string "
+           "with reversing the preference of learning. "),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY,
+        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
+        N_("Commit the _first segment:"),
+        N_("Select keys to commit the first segment"),
+        N_("The key events to commit the first segment "
+           "with reversing the preference of learning. "),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY,
+        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
+        N_("Commit the _selected segment:"),
+        N_("Select keys to commit the selected segment"),
+        N_("The key events to commit the selected segment "
+           "with reversing the preference of learning. "),
+        NULL,
+        false,
+    },
     {
         SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND,
         SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND_DEFAULT,
@@ -476,6 +509,16 @@ StringConfigData config_keyboards_edit [] =
         NULL,
         false,
     },
+    {
+        SCIM_ANTHY_CONFIG_CANCEL_ALL_KEY,
+        SCIM_ANTHY_CONFIG_CANCEL_ALL_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_CANCEL_ALL_KEY_DEFAULT,
+        N_("Cancel all"),
+        N_("Select cancel all keys"),
+        N_("The key events to return to initial state. "),
+        NULL,
+        false,
+    },
     {
         SCIM_ANTHY_CONFIG_DO_NOTHING_KEY,
         SCIM_ANTHY_CONFIG_DO_NOTHING_KEY_DEFAULT,
@@ -837,6 +880,26 @@ StringConfigData config_keyboards_direct_select [] =
 
 StringConfigData config_keyboards_converting [] =
 {
+    {
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_FORWARD_KEY,
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_FORWARD_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_FORWARD_KEY_DEFAULT,
+        N_("Convert character type to forward"),
+        N_("Select keys to convert characgter type to forward"),
+        N_("Rotate character type forward."),
+        NULL,
+        false,
+    },
+    {
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_BACKWARD_KEY,
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_BACKWARD_KEY_DEFAULT,
+        SCIM_ANTHY_CONFIG_CONV_CHAR_TYPE_BACKWARD_KEY_DEFAULT,
+        N_("Convert character type to backward"),
+        N_("Select keys to convert characgter type to backward"),
+        N_("Rotate character type backward."),
+        NULL,
+        false,
+    },
     {
         SCIM_ANTHY_CONFIG_CONV_TO_HIRAGANA_KEY,
         SCIM_ANTHY_CONFIG_CONV_TO_HIRAGANA_KEY_DEFAULT,
@@ -1017,53 +1080,6 @@ StringConfigData config_keyboards_dict [] =
     },
 };
 
-StringConfigData config_keyboards_reverse_learning [] =
-{
-    {
-        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY,
-        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY_DEFAULT,
-        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY_DEFAULT,
-        N_("_Commit:"),
-        N_("Select commit keys"),
-        N_("The key events to commit the preedit string "
-           "with reversing the preference of learning. "),
-        NULL,
-        false,
-    },
-    {
-        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY,
-        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
-        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
-        N_("Commit the _first segment:"),
-        N_("Select keys to commit the first segment"),
-        N_("The key events to commit the first segment "
-           "with reversing the preference of learning. "),
-        NULL,
-        false,
-    },
-    {
-        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY,
-        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
-        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY_DEFAULT,
-        N_("Commit the _selected segment:"),
-        N_("Select keys to commit the selected segment"),
-        N_("The key events to commit the selected segment "
-           "with reversing the preference of learning. "),
-        NULL,
-        false,
-    },
-    {
-        NULL,
-        "",
-        "",
-        NULL,
-        NULL,
-        NULL,
-        NULL,
-        false,
-    },
-};
-
 ColorConfigData config_color_common [] =
 {
     {
