@@ -69,7 +69,7 @@ private:
 class Reading
 {
 public:
-    Reading (AnthyInstance &anthy, Key2KanaTableSet & tables);
+    Reading (AnthyInstance &anthy);
     virtual ~Reading ();
 
     bool         can_process_key_event (const KeyEvent & key);
@@ -77,47 +77,53 @@ public:
     void         finish                (void);
     void         clear                 (void);
 
-    WideString   get                   (unsigned int start  = 0,
-                                        int          length = -1,
-                                        StringType   type
+    WideString   get                   (unsigned int     start  = 0,
+                                        int              length = -1,
+                                        StringType       type
                                         = SCIM_ANTHY_STRING_HIRAGANA);
-    void         get                   (WideString & string,
-                                        unsigned int start  = 0,
-                                        int          length = -1,
-                                        StringType   type
+    void         get                   (WideString     & string,
+                                        unsigned int     start  = 0,
+                                        int              length = -1,
+                                        StringType       type
                                         = SCIM_ANTHY_STRING_HIRAGANA);
-    String       get_raw               (unsigned int start  = 0,
-                                        int          length = -1);
-    void         get_raw               (String     & string,
-                                        unsigned int start  = 0,
-                                        int          length = -1);
-    void         erase                 (unsigned int start  = 0,
-                                        int          length = -1,
-                                        bool         allow_split = false);
+    String       get_raw               (unsigned int     start  = 0,
+                                        int              length = -1);
+    void         get_raw               (String         & string,
+                                        unsigned int     start  = 0,
+                                        int              length = -1);
+    void         erase                 (unsigned int     start  = 0,
+                                        int              length = -1,
+                                        bool             allow_split = false);
 
     unsigned int get_length            (void);
     unsigned int get_caret_pos         (void);
-    void         set_caret_pos         (unsigned int pos);
-    void         move_caret            (int          step,
-                                        bool         allow_split = false);
+    void         set_caret_pos         (unsigned int     pos);
+    void         move_caret            (int              step,
+                                        bool             allow_split = false);
 
-    void         set_ten_key_type      (TenKeyType   type);
-    TenKeyType   get_ten_key_type      (void);
-
-    void         set_typing_method     (TypingMethod   method,
-                                        Key2KanaTable *fundamental_table
-                                        = NULL);
+    void         set_typing_method     (TypingMethod     method);
     TypingMethod get_typing_method     (void);
+    void         set_period_style      (PeriodStyle      style);
+    PeriodStyle  get_period_style      (void);
+    void         set_comma_style       (CommaStyle       style);
+    CommaStyle   get_comma_style       (void);
+    void         set_symbol_width      (bool             half);
+    bool         get_symbol_width      (void);
+    void         set_number_width      (bool             half);
+    bool         get_number_width      (void);
 
 private:
     void         reset_pending         (void);
-    void         split_segment         (unsigned int seg_id);
+    void         split_segment         (unsigned int     seg_id);
 
 private:
     AnthyInstance         &m_anthy;
 
+    // tables
+    Key2KanaTableSet       m_key2kana_tables;
+    Key2KanaTableSet       m_nicola_tables;
+
     // convertors
-    Key2KanaTableSet      &m_key2kana_tables;
     Key2KanaConvertor      m_key2kana_normal;
     KanaConvertor          m_kana;
     NicolaConvertor        m_nicola;
