@@ -191,8 +191,11 @@ static struct KeyboardConfigPage __key_conf_pages[] =
 };
 static unsigned int __key_conf_pages_num = sizeof (__key_conf_pages) / sizeof (KeyboardConfigPage);
 
-const int INDEX_SEARCH_BY_KEY = __key_conf_pages_num;
-const int INDEX_ALL           = __key_conf_pages_num + 1;
+const int KEY_CATEGORY_INDEX_SEARCH_BY_KEY = __key_conf_pages_num;
+const int KEY_CATEGORY_INDEX_ALL           = __key_conf_pages_num + 1;
+
+const int KEY_THEME_INDEX_USER_DEFINED = 0;
+const int KEY_THEME_INDEX_DEFAULT      = 1;
 
 static ComboConfigCandidate input_modes[] =
 {
@@ -277,10 +280,8 @@ static void     on_default_toggle_button_toggled  (GtkToggleButton  *togglebutto
                                                    gpointer          user_data);
 static void     on_default_spin_button_changed    (GtkSpinButton    *spinbutton,
                                                    gpointer          user_data);
-#if 0
 static void     on_default_key_selection_clicked  (GtkButton        *button,
                                                    gpointer          user_data);
-#endif
 static void     on_default_combo_changed          (GtkEditable      *editable,
                                                    gpointer          user_data);
 static void     on_default_option_menu_changed    (GtkOptionMenu    *omenu,
@@ -341,7 +342,7 @@ find_int_config_entry (const char *config_key)
     return NULL;
 }
 
-StringConfigData *
+static StringConfigData *
 find_string_config_entry (const char *config_key)
 {
     if (!config_key)
@@ -408,12 +409,12 @@ create_check_button (const char *config_key)
     return GTK_WIDGET (entry->widget);
 }
 
-void
+GtkWidget *
 create_spin_button (const char *config_key, GtkTable *table, int idx)
 {
     IntConfigData *entry = find_int_config_entry (config_key);
     if (!entry)
-        return;
+        return NULL;
 
     GtkWidget *label = gtk_label_new_with_mnemonic (_(entry->label));
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
@@ -457,38 +458,44 @@ create_spin_button (const char *config_key, GtkTable *table, int idx)
     if (entry->tooltip)
         gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (entry->widget),
                               _(entry->tooltip), NULL);
+
+    return GTK_WIDGET (entry->widget);
 }
 
 GtkWidget *
-create_entry (StringConfigData *data, GtkTable *table, int idx)
+create_entry (const char *config_key, GtkTable *table, int idx)
 {
+    StringConfigData *entry = find_string_config_entry (config_key);
+    if (!entry)
+        return NULL;
+
     GtkWidget *label = gtk_label_new (NULL);
-    gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _(data->label));
+    gtk_label_set_text_with_mnemonic (GTK_LABEL (label), _(entry->label));
     gtk_widget_show (label);
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_misc_set_padding (GTK_MISC (label), 4, 0);
     gtk_table_attach (GTK_TABLE (table), label, 0, 1, idx, idx + 1,
                       (GtkAttachOptions) (GTK_FILL),
                       (GtkAttachOptions) (GTK_FILL), 4, 4);
-    (data)->widget = gtk_entry_new ();
+    (entry)->widget = gtk_entry_new ();
     gtk_label_set_mnemonic_widget (GTK_LABEL (label),
-                                   GTK_WIDGET (data->widget));
-    g_signal_connect ((gpointer) (data)->widget, "changed",
+                                   GTK_WIDGET (entry->widget));
+    g_signal_connect ((gpointer) (entry)->widget, "changed",
                       G_CALLBACK (on_default_editable_changed),
-                      data);
-    gtk_widget_show (GTK_WIDGET (data->widget));
-    gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (data->widget),
+                      entry);
+    gtk_widget_show (GTK_WIDGET (entry->widget));
+    gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (entry->widget),
                       1, 2, idx, idx + 1,
                       (GtkAttachOptions) (GTK_FILL|GTK_EXPAND),
                       (GtkAttachOptions) (GTK_FILL), 4, 4);
 
     if (!__widget_tooltips)
         __widget_tooltips = gtk_tooltips_new();
-    if (data->tooltip)
-        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (data->widget),
-                              _(data->tooltip), NULL);
+    if (entry->tooltip)
+        gtk_tooltips_set_tip (__widget_tooltips, GTK_WIDGET (entry->widget),
+                              _(entry->tooltip), NULL);
 
-    return GTK_WIDGET (data->widget);
+    return GTK_WIDGET (entry->widget);
 }
 
 GtkWidget *
@@ -593,6 +600,24 @@ create_option_menu (const char *config_key, gpointer candidates_p,
     return GTK_WIDGET (entry->widget);
 }
 
+GtkWidget *
+create_key_select_button (const char *config_key, GtkTable *table, int idx)
+{
+    StringConfigData *entry = find_string_config_entry (config_key);
+    if (!entry)
+        return NULL;
+
+    GtkWidget *button = gtk_button_new_with_label ("...");
+    gtk_widget_show (button);
+    gtk_table_attach (GTK_TABLE (table), button, 2, 3, idx, idx + 1,
+                      GTK_FILL, GTK_FILL, 4, 4);
+    g_signal_connect ((gpointer) button, "clicked",
+                      G_CALLBACK (on_default_key_selection_clicked),
+                      entry);
+
+    return button;
+}
+
 GtkWidget *
 create_color_button (const char *config_key)
 {
@@ -712,7 +737,8 @@ key_list_view_popup_key_selection (GtkTreeView *treeview)
                                     COLUMN_VALUE, data->value.c_str(),
                                     -1);
                 gtk_option_menu_set_history (
-                    GTK_OPTION_MENU (__widget_key_theme_menu), 0);
+                    GTK_OPTION_MENU (__widget_key_theme_menu),
+                    KEY_THEME_INDEX_USER_DEFINED);
                 data->changed = true;
                 __config_changed = true;
             }
@@ -927,7 +953,7 @@ static GtkWidget *
 create_learning_page ()
 {
     GtkWidget *vbox, *vbox2, *hbox, *alignment, *table;
-    GtkWidget *widget, *label, *button;
+    GtkWidget *widget, *label;
 
     vbox = gtk_vbox_new (FALSE, 0);
     gtk_widget_show (vbox);
@@ -978,18 +1004,22 @@ create_learning_page ()
     gtk_container_add (GTK_CONTAINER (alignment), table);
     gtk_widget_show (table);
 
-    for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++) {
-        StringConfigData *entry = &config_keyboards_reverse_learning[i];
-        widget = create_entry (entry, GTK_TABLE (table), i);
+    StringConfigData *entries[3];
+    entries [0] = find_string_config_entry (
+        SCIM_ANTHY_CONFIG_COMMIT_REVERSE_LEARN_KEY);
+    entries [1] = find_string_config_entry (
+        SCIM_ANTHY_CONFIG_COMMIT_FIRST_SEGMENT_REVERSE_LEARN_KEY);
+    entries [2] = find_string_config_entry (
+        SCIM_ANTHY_CONFIG_COMMIT_SELECTED_SEGMENT_REVERSE_LEARN_KEY);
+
+    for (unsigned int i = 0;
+         i < sizeof (entries) / sizeof (StringConfigData*);
+         i++)
+    {
+        StringConfigData *entry = entries[i];
+        widget = create_entry (entry->key, GTK_TABLE (table), i);
         gtk_entry_set_editable (GTK_ENTRY (widget), FALSE);
-        button = gtk_button_new_with_label ("...");
-        gtk_widget_show (button);
-        gtk_table_attach (GTK_TABLE (table), button, 2, 3, i, i + 1,
-                          GTK_FILL, GTK_FILL, 4, 4);
-        gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-        g_signal_connect ((gpointer) button, "clicked",
-                          G_CALLBACK (on_default_key_selection_clicked),
-                          entry);
+        create_key_select_button (entry->key, GTK_TABLE (table), i);
     }
 
     return vbox;
@@ -1005,8 +1035,9 @@ create_dict_page (void)
     gtk_widget_show (table);
 
     // dict admin command
+    create_entry (SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND,
+                  GTK_TABLE (table), 0);
     entry = find_string_config_entry (SCIM_ANTHY_CONFIG_DICT_ADMIN_COMMAND);
-    create_entry (entry, GTK_TABLE (table), 0);
 
     button = gtk_button_new_with_mnemonic (_("_Launch"));
     gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (button),
@@ -1018,8 +1049,9 @@ create_dict_page (void)
     gtk_widget_show (button);
 
     // add word command
+    create_entry (SCIM_ANTHY_CONFIG_ADD_WORD_COMMAND,
+                  GTK_TABLE (table), 1);
     entry = find_string_config_entry (SCIM_ANTHY_CONFIG_ADD_WORD_COMMAND);
-    create_entry (entry, GTK_TABLE (table), 1);
 
     button = gtk_button_new_with_mnemonic (_("_Launch"));
     gtk_table_attach (GTK_TABLE (table), GTK_WIDGET (button),
@@ -1336,12 +1368,14 @@ setup_key_theme_menu (GtkOptionMenu *omenu)
                                      (gpointer) (on_key_theme_menu_changed),
                                      NULL);
 
-    gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 1);
+    gtk_option_menu_set_history (GTK_OPTION_MENU (omenu),
+                                 KEY_THEME_INDEX_DEFAULT);
 
     if (__config_key_theme_file == __user_style_file.get_file_name () ||
         __config_key_theme      == __user_style_file.get_title ())
     {
-        gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), 0);
+        gtk_option_menu_set_history (GTK_OPTION_MENU (omenu),
+                                     KEY_THEME_INDEX_USER_DEFINED);
 
     } else {
         GList *node, *list = gtk_container_get_children (GTK_CONTAINER (menu));
@@ -1402,14 +1436,6 @@ setup_widget_value (void)
             }
         }
     }
-
-    for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++) {
-        if (config_keyboards_reverse_learning[i].widget) {
-            gtk_entry_set_text (
-                GTK_ENTRY (config_keyboards_reverse_learning[i].widget),
-                config_keyboards_reverse_learning[i].value.c_str ());
-        }
-    }
     
     for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
         ColorConfigData &entry = config_color_common[i];
@@ -1420,13 +1446,15 @@ setup_widget_value (void)
     }
 
     gtk_option_menu_set_history
-        (GTK_OPTION_MENU (__widget_key_categories_menu), 0);
+        (GTK_OPTION_MENU (__widget_key_categories_menu),
+         KEY_CATEGORY_INDEX_ALL);
+    GtkTreeView *treeview = GTK_TREE_VIEW (__widget_key_list_view);
+    GtkListStore *store = GTK_LIST_STORE (gtk_tree_view_get_model (treeview));
+    gtk_list_store_clear (store);
+    for (unsigned int i = 0; i < __key_conf_pages_num; i++)
+        append_key_bindings (treeview, i, NULL);
     gtk_widget_set_sensitive (__widget_key_filter, FALSE);
     gtk_widget_set_sensitive (__widget_key_filter_button, FALSE);
-    GtkTreeModel *model;
-    model = gtk_tree_view_get_model (GTK_TREE_VIEW (__widget_key_list_view));
-    gtk_list_store_clear (GTK_LIST_STORE (model));
-    append_key_bindings (GTK_TREE_VIEW (__widget_key_list_view), 0, NULL);
 
     // setup option menu
     setup_key_theme_menu (GTK_OPTION_MENU (__widget_key_theme_menu));
@@ -1508,11 +1536,6 @@ load_config (const ConfigPointer &config)
         }
     }
 
-    for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++) {
-        StringConfigData &entry = config_keyboards_reverse_learning[i];
-        entry.value = config->read (String (entry.key), entry.value);
-    }
-
     for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
         ColorConfigData &entry = config_color_common[i];
         entry.fg_value = config->read (String (entry.fg_key), entry.fg_value);
@@ -1538,9 +1561,6 @@ load_config (const ConfigPointer &config)
             __key_conf_pages[j].data[i].changed = false;
     }
 
-    for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++)
-        config_keyboards_reverse_learning[i].changed = false;
-
     for (unsigned int i = 0; config_color_common[i].fg_key; i++)
         config_color_common[i].changed = false;
 
@@ -1590,18 +1610,13 @@ save_config (const ConfigPointer &config)
         }
     }
 
-    for (unsigned int i = 0; config_keyboards_reverse_learning[i].key; i++) {
-        StringConfigData &entry = config_keyboards_reverse_learning[i];
-        if (entry.changed)
-            entry.value = config->write (String (entry.key), entry.value);
-        entry.changed = false;
-    }
-
     for (unsigned int i = 0; config_color_common[i].fg_key; i++) {
         ColorConfigData &entry = config_color_common[i];
         if (entry.changed) {
-            entry.fg_value = config->write (String (entry.fg_key), entry.fg_value);
-            entry.bg_value = config->write (String (entry.bg_key), entry.bg_value);
+            entry.fg_value = config->write (String (entry.fg_key),
+                                            entry.fg_value);
+            entry.bg_value = config->write (String (entry.bg_key),
+                                            entry.bg_value);
         }
         entry.changed = false;
     }
@@ -1667,7 +1682,7 @@ on_default_editable_changed (GtkEditable *editable,
     }
 }
 
-void
+static void
 on_default_key_selection_clicked (GtkButton *button,
                                   gpointer   user_data)
 {
@@ -1793,14 +1808,14 @@ on_key_category_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
     if (idx >= 0 && idx < (gint) __key_conf_pages_num) {
         append_key_bindings (treeview, idx, NULL);
 
-    } else if (idx == INDEX_SEARCH_BY_KEY) {
+    } else if (idx == KEY_CATEGORY_INDEX_SEARCH_BY_KEY) {
         // search by key
         use_filter = true;
         const char *str = gtk_entry_get_text (GTK_ENTRY (__widget_key_filter));
         for (unsigned int i = 0; i < __key_conf_pages_num; i++)
             append_key_bindings (treeview, i, str);
 
-    } else if (idx == INDEX_ALL) {
+    } else if (idx == KEY_CATEGORY_INDEX_ALL) {
         // all
         for (unsigned int i = 0; i < __key_conf_pages_num; i++)
             append_key_bindings (treeview, i, NULL);
@@ -1835,11 +1850,11 @@ on_key_theme_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
     }
 
     // set new key bindings
-    if (idx == 0) {
+    if (idx == KEY_THEME_INDEX_USER_DEFINED) {
         __config_key_theme      = String ("User defined");
         __config_key_theme_file = String (""); //__user_style_file.get_file_name ();
 
-    } else if (idx == 1) {
+    } else if (idx == KEY_THEME_INDEX_DEFAULT) {
         for (unsigned int j = 0; j < __key_conf_pages_num; j++) {
             for (unsigned int i = 0; __key_conf_pages[j].data[i].key; i++) {
                 __key_conf_pages[j].data[i].value
@@ -1873,16 +1888,24 @@ on_key_theme_menu_changed (GtkOptionMenu *omenu, gpointer user_data)
     }
 
     // sync widgets
-    if (idx != 0) {
+    if (idx != KEY_THEME_INDEX_USER_DEFINED) {
         gtk_option_menu_set_history
-            (GTK_OPTION_MENU (__widget_key_categories_menu), 0);
+            (GTK_OPTION_MENU (__widget_key_categories_menu),
+             KEY_CATEGORY_INDEX_ALL);
         gtk_widget_set_sensitive (__widget_key_filter, FALSE);
         gtk_widget_set_sensitive (__widget_key_filter_button, FALSE);
         GtkTreeModel *model;
         model = gtk_tree_view_get_model (
             GTK_TREE_VIEW (__widget_key_list_view));
         gtk_list_store_clear (GTK_LIST_STORE (model));
-        append_key_bindings (GTK_TREE_VIEW (__widget_key_list_view), 0, NULL);
+#if 0
+        append_key_bindings (GTK_TREE_VIEW (__widget_key_list_view),
+                             KEY_CATEGORY_INDEX_ALL, NULL);
+#else
+        for (unsigned int i = 0; i < __key_conf_pages_num; i++)
+            append_key_bindings (GTK_TREE_VIEW (__widget_key_list_view),
+                                 i, NULL);
+#endif
     }
 
     __config_changed = true;
